package cn.fastfun.service.impl;


import cn.fastfun.controller.dto.LoginUser;
import cn.fastfun.service.UtilService;
import cn.fastfun.service.entity.SysUser;
import cn.fastfun.service.repository.SysUserRepository;
import cn.fastfun.util.JwtUtil;
import com.bridge.dto.ApiDTO;
import com.bridge.exception.ApiRuntimeException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.data.domain.Example;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

@Slf4j
@Service
public class UtilServiceImpl implements UtilService {


    @Autowired
    Environment environment;

    @Resource
    SysUserRepository userRepository;

    @Autowired
    RedisTemplate redisTemplate;

    String secretKey;

    String tokenTime;


    Map<String, Integer> loginFailed = new ConcurrentHashMap<>();// 登录错误次数
    Map<String, Long> loginFailedTime = new ConcurrentHashMap<>();// 最后登录时间

    @Override
    public LoginUser getUser() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        if (null == request.getHeader("X-Token")) return new LoginUser();
        // 获得token
        Map<String, Object> jwt = JwtUtil.parseJWT(request.getHeader("X-Token"), getSecretKey());
        LoginUser loginUser = getUserForRedis(String.valueOf(jwt.get("userId")));
        loginUser.setUserName(String.valueOf(jwt.get("userName")));
        if (StringUtils.isEmpty(loginUser.getLoginTime())) {
            clearLoginUser(loginUser.getId());
            throw new ApiRuntimeException(501, "账号信息已经更新，请重新登录");
        }
        if (loginUser.getLoginTime().getTime() != Long.parseLong(String.valueOf(jwt.get("loginTime"))))
            throw new ApiRuntimeException(501, "账号信息已经更新，请重新登录");
        return loginUser;
    }

    private void setLoginUser(LoginUser loginUser) {
        try {
            // 缓存到redis中
            redisTemplate.opsForValue().set(getSecretKey().concat("_user_info_").concat(loginUser.getId()), new ObjectMapper().writeValueAsString(loginUser), getTokenTime(), TimeUnit.SECONDS);
        } catch (JsonProcessingException e) {
            log.error("Login JSON Error");
        }

    }

    @Override
    public void clearLoginUser(String userId) {
        redisTemplate.delete(getSecretKey().concat("_user_info_").concat(userId));
    }

    @Override
    public ApiDTO login(String userName, String userPass) {
        if (loginFailedTime.containsKey(userName) && (System.currentTimeMillis() / 1000 - loginFailedTime.get(userName)) > 300) {
            loginFailedTime.remove(userName);
            loginFailed.remove(userName);
        }
        if (loginFailed.containsKey(userName) && loginFailed.get(userName) >= 5 && (System.currentTimeMillis() / 1000 - loginFailedTime.get(userName)) < 300) {
            return ApiDTO.error(503, "密码超过错误次数，账号被锁定！");
        }
        SysUser t = new SysUser();
        t.setUserName(userName);
        t.setUserPass(userPass);
        Optional<SysUser> optional = userRepository.findOne(Example.of(t));


        // 手机号登录
        SysUser t2 = new SysUser();
        t2.setMobile(userName);
        t2.setUserPass(userPass);
        Optional<SysUser> optional2 = userRepository.findOne(Example.of(t2));



        if (!optional.isPresent() && !optional2.isPresent()) {
            if (!loginFailed.containsKey(userName)) loginFailed.put(userName, 0);
            loginFailed.put(userName, loginFailed.get(userName) + 1);
            loginFailedTime.put(userName, System.currentTimeMillis() / 1000);
            throw new ApiRuntimeException(501, "账号或密码错误！");
        }
        t = optional.isPresent()?optional.get(): optional2.get();
        if (null != t && !StringUtils.isEmpty(t.getId())) {
            // 生成jwt token
            Map<String, Object> map = new HashMap<>();
            Object data = redisTemplate.opsForValue().get(getSecretKey().concat("_user_info_").concat(t.getId()));
            if (!StringUtils.isEmpty(data)) {
                try {
                    LoginUser loginUser = new ObjectMapper().readValue(String.valueOf(data), LoginUser.class);
                    Date date = new Date();
//                    resetLoginUser(t, date);
                    map.put("loginTime", loginUser.getLoginTime().getTime());
                } catch (IOException e) {
                    log.error("JSON 转换错误!", e);
                }
            } else {
                Date date = new Date();
                resetLoginUser(t, date);
                map.put("loginTime", date.getTime());
            }
            map.put("userId", t.getId());
            map.put("data", new Date());
            map.put("userName", t.getUserName());
            if (loginFailed.containsKey(userName)) {
                loginFailed.remove(userName);
                loginFailedTime.remove(userName);
            }
            return ApiDTO.ok("成功登陆", JwtUtil.createJWT(map, getSecretKey()));
        }
        return ApiDTO.error(501, "账号或密码错误");
    }

    @Override
    public void resetLoginUser(SysUser sysUser, Date date) {
        LoginUser loginUser = new LoginUser();
        loginUser.setLoginTime(date);
        loginUser.setUser(sysUser);

        setLoginUser(loginUser);
    }

    @Override
    public String getUserId() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        // 获得token
        Map<String, Object> jwt = JwtUtil.parseJWT(request.getHeader("X-Token"), getSecretKey());
        return String.valueOf(jwt.get("userId"));
    }

    @Override
    public String getUserName() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        // 获得token
        Map<String, Object> jwt = JwtUtil.parseJWT(request.getHeader("X-Token"), getSecretKey());
        return String.valueOf(jwt.get("userName"));
    }

    private LoginUser getUserForRedis(String userId) {
        if (StringUtils.isEmpty(userId))
            throw new ApiRuntimeException(501, "账号过期");
        Object data = redisTemplate.opsForValue().get(getSecretKey().concat("_user_info_").concat(userId));
        if (StringUtils.isEmpty(data))
            throw new ApiRuntimeException(501, "账号过期");
        try {
            LoginUser loginUser = new ObjectMapper().readValue(String.valueOf(data), LoginUser.class);
            return loginUser;
        } catch (IOException e) {
            log.error("账号信息错误!", e);
            throw new ApiRuntimeException(501, "账号过期");
        }
    }

    private String getSecretKey() {
        if (!StringUtils.isEmpty(secretKey))
            return secretKey;
        // 获取配置密钥
        secretKey = environment.getProperty("project.secret-key");
        if (StringUtils.isEmpty(secretKey)) {
            log.error("系统未配置secret-key");
            throw new ApiRuntimeException("系统未配置secret-key");
        }
        return secretKey;
    }

    private Long getTokenTime() {
        if (!StringUtils.isEmpty(tokenTime))
            return Long.parseLong(tokenTime);
        // 取配置缓存时间
        tokenTime = environment.getProperty("project.token-time");
        long cacheTime = 3600;
        if (StringUtils.isEmpty(tokenTime))
            cacheTime = Long.parseLong(tokenTime);
        return cacheTime;
    }

}
